package greenfoot.junitUtils;

import greenfoot.*;
import greenfoot.core.WorldHandler;
import greenfoot.junitUtils.WorldCreator;
import greenfoot.junitUtils.runner.TestRunnerState;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

/**
 * It cleans queues used for testing. If we wants to verify the behaviour 
 * of any scenario, a world is needed and greenfoot objects (world and 
 * actors) must execute theirs acts. When a same test need many worlds or
 * many act cycles, these worlds can be created by getWorld(...) methods 
 * and acts can be executed by runOnce(...) methods with empty queues. Also, 
 * some events (start, stop, delay, ...) of a tested scenario are simulated 
 * using others static methods.
 *
 * @author Francisco Guerra (francisco.guerra@ulpgc.es)
 */
public class WorldCreator {
	
	/**
	 * It creates and activates a generic world for testing.
	 * It should be used instead of "new World ()" when the 
	 * world of the stage has not yet been programmed, or when 
	 * an empty world is needed (without actors) or when a 
	 * world needs to be created and the data used in its 
	 * creation should not affect the rest of the test.
	 * When more than one world is created in the same test, 
	 * the first created world will be the world associated 
	 * with the scenario, and otherWorld can be activated by 
	 * calling the Greenfoot.setWorld(otherWorld).
	 * 
	 * 
	 * For the next actor behaviour:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>public class</b> ExampleActorGetWorld <b>extends</b> Actor {
	 *         <i>@Override</i>
	 *         <b>public void</b> act(){
	 *             move(1);
	 *         }
	 *     } 
	 * </code>
	 * </pre>
	 * 
	 * The next tests verify that the scenario only runs one world at a time:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>import</b> ...
	 *     <b>import</b> greenfoot.TestRunnerState.WorldCreator;
	 *     
	 *     <i>@RunWith</i>(GreenfootRunner.class)
	 *     <b>public class</b> GetWorldTest {
	 *         World world1;
	 *         World world2;
	 *         ExampleActorGetWorld actor1;
	 *         ExampleActorGetWorld actor2;
	 *         
	 *         <i>@Before</i>
	 *         <b>public void</b> setUp() <b>throws</b> Exception {
	 *             <b>// GIVEN </b>
	 *             world1 = WorldCreator.getWorld(15, 15, 60);
	 *             world2 = WorldCreator.getWorld(15, 15, 60);
	 *             actor1 = <b>new</b> ExampleActorGetWorld();
	 *             world1.addObject(actor1, 3, 5);
	 *             actor2 = <b>new</b> ExampleActorGetWorld();
	 *             world2.addObject(actor2, 3, 5);
	 *         }
	 *         
	 *         <i>@Test</i>
	 *         <b>public void</b> testGetWorld() <b>throws</b> Exception {
	 *             <b>// WHEN </b>
	 *             WorldCreator.runOnce();
	 *             
	 *             <b>// THEN </b>
	 *             <i>assertEquals</i>( 4, actor1.getX());
	 *             <i>assertEquals</i>( 3, actor2.getX());
	 *         }
	 *         
	 *         <i>@Test</i>
	 *         <b>public void</b> testSetWorld() <b>throws</b> Exception {
	 *             <b>// WHEN </b>
	 *             Greenfoot.setWorld(world2);
	 *             WorldCreator.runOnce();
	 *             
	 *             <b>// THEN </b>
	 *             <i>assertEquals</i>( 3, actor1.getX());
	 *             <i>assertEquals</i>( 4, actor2.getX());
	 *         }
	 *     }
	 * </code>
	 * </pre>
	 * 
	 * @param worldWidth represents the world width
	 * 
	 * @param worldHeight represents the world height
	 * 
	 * @param cellSize represents the cell size
	 * 
	 * @return the created world
	 * 
     * @throws GreenfootRunnerExpected is raised when <i>@RunWith</i>(GreenfootRunner.class) 
     *         or <i>@ExtendWith</i>(GreenfootRunner.class) is not found.
	 */
    public synchronized static World getWorld(int worldWidth, int worldHeight, int cellSize) throws GreenfootRunnerExpected {
    	TestRunnerState.assertRunningUnitTest();
        
        beginCycle();
        World world = new World (worldWidth, worldHeight, cellSize){};
        endCycle();
        
        return world;
    }

	/**
	 * It creates and activates a user world for testing. 
	 * It should be used instead of "new World ()" when a 
	 * world needs to be created and the data used in its 
	 * creation should not affect the rest of the test.
	 * When more than one world is created in the same test, 
	 * the first created world will be the world associated 
	 * with the scenario, and otherWorld can be activated by 
	 * calling the Greenfoot.setWorld(otherWorld).
	 * 
	 * 
	 * For the next actor behaviour:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>public class</b> ExampleWorld <b>extends</b> World {
	 *         <b>private int</b> value = 0;
	 *         
	 *         <b>public</b> ExampleWorld(){
	 *             <b>super</b>(15, 15, 60);
	 *         }
	 *         
	 *         <b>public int</b> getValue(){
	 *             <b>return</b> value;
	 *         }
	 *         
	 *         <i>@Override</i>
	 *         <b>public void</b> act(){
	 *             value = 1;
	 *         }
	 *     } 
	 * </code>
	 * </pre>
	 * 
	 * The next tests verify that the scenario only runs one world at a time:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>import</b> ...
	 *     <b>import</b> greenfoot.TestRunnerState.WorldCreator;
	 *     
	 *     <i>@RunWith</i>(GreenfootRunner.class)
	 *     <b>public class</b> GetWorldTest {
	 *         ExampleWorld world1;
	 *         ExampleWorld world2;
	 *         
	 *         <i>@Before</i>
	 *         <b>public void</b> setUp() <b>throws</b> Exception {
	 *             <b>// GIVEN </b>
	 *             world1 = WorldCreator.getWorld(ExampleWorld.<b>class</b>);
	 *             world2 = WorldCreator.getWorld(ExampleWorld.<b>class</b>);
	 *         }
	 *         
	 *         <i>@Test</i>
	 *         <b>public void</b> testGetWorld() <b>throws</b> Exception {
	 *             <b>// WHEN </b>
	 *             WorldCreator.runOnce();
	 *             
	 *             <b>// THEN </b>
	 *             <i>assertEquals</i>( 1, world1.getValue());
	 *             <i>assertEquals</i>( 0, world2.getValue());
	 *         }
	 *         
	 *         <i>@Test</i>
	 *         <b>public void</b> testSetWorld() <b>throws</b> Exception {
	 *             <b>// WHEN </b>
	 *             Greenfoot.setWorld(world2);
	 *             WorldCreator.runOnce();
	 *             
	 *             <b>// THEN </b>
	 *             <i>assertEquals</i>( 0, world1.getValue());
	 *             <i>assertEquals</i>( 1, world2.getValue());
	 *         }
	 *     }
	 * </code>
	 * </pre>
	 * 
     * @param <worldType> represents the user world class
     *
	 * @param worldType represents the user world class. 
	 * 
	 * @param parameters is the parameters list of constructor. 
	 * 
     * @return the created world.
     *         
     * @throws GreenfootRunnerExpected is raised when <i>@RunWith</i>(GreenfootRunner.class) 
     *         or <i>@ExtendWith</i>(GreenfootRunner.class) is not found.
     *         
	 * @throws WorldConstructorNotFound is raise when there is not any contructor of the world class that has a 
	 *                                  parameter list that matches with parameters.
     */
    public synchronized static <worldType> worldType getWorld(Class<? extends World> worldType, Object ... parameters)
    		throws GreenfootRunnerExpected, WorldConstructorNotFound {        
    	TestRunnerState.assertRunningUnitTest();

        Constructor<? extends World> constructor = findConstructor(worldType, parameters);
        
        beginCycle();        
        worldType world = getWorld(worldType, constructor, parameters);       
        endCycle();
        
        return world;	        
    }

	@SuppressWarnings("unchecked")
	private static <worldType> worldType getWorld(Class<? extends World> worldType, Constructor<? extends World> constructor, Object... parameters)
			throws WorldConstructorNotFound {
		try {
			return (worldType) constructor.newInstance(parameters);
		} catch (InstantiationException | IllegalAccessException | IllegalArgumentException
				| InvocationTargetException e) {
			e.printStackTrace();
			throw new WorldConstructorNotFound();
		}
	}
    
    @SuppressWarnings({ "rawtypes", "unchecked" })
    private static Constructor<? extends World> findConstructor(Class<? extends World> worldType, 
    		Object[] parameters) throws WorldConstructorNotFound{
        if (parameters == null) {
            parameters = new Object[0];
        }
        for(Constructor constructor : worldType.getDeclaredConstructors()){
			Class[] types = constructor.getParameterTypes();
            if(types.length==parameters.length) {
	            if (match(parameters, types)) {
	                return constructor;
	            }
            }
        }
        throw new WorldConstructorNotFound();
    }

    @SuppressWarnings({ "rawtypes", "unchecked" })
	private static boolean match(Object[] parameters, Class[] types) {
		for(int i = 0; i < types.length; i++){
		    Class need = types[i];
		    Class<? extends Object> got = parameters[i].getClass();
		    if(!need.isAssignableFrom(got)){
		        if(need.isPrimitive()){
					if (!(int.class.equals(need) && Integer.class.equals(got))
							|| (long.class.equals(need) && Long.class.equals(got))
							|| (char.class.equals(need) && Character.class.equals(got))
							|| (short.class.equals(need) && Short.class.equals(got))
							|| (boolean.class.equals(need) && Boolean.class.equals(got))
							|| (byte.class.equals(need) && Byte.class.equals(got))) {
						return false;
		            }
		            
		        }else{
		            return false;
		        }
		    }
		}
		return true;
	}   
    
    /**
     * It executes one acts cycle (the act of world and the acts 
     * of actors added to current world of the scenario). Delay, 
     * Speed and SoundPlayed events are deleted before acts are 
     * executed and Mic Level, Random and Ask events are deleted 
     * after acts are executed. Also, the keyboard and mouse 
     * buffers are reset after this operation.
     * 
	 * For the next actor behaviour:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>public class</b> ExampleActorRunOnce <b>extends</b> Actor {
	 *         <i>@Override</i>
	 *         <b>public void</b> act(){
	 *             <b>if</b> (isTouching(ExampleActorRunOnce.<b>class</b>)){
	 *                 move(4);
	 *             } <b>else</b> {
	 *                 move(2);
	 *             }
	 *         }
	 *     } 
	 * </code>
	 * </pre>
	 * 
	 * The next test verifies that only the act method of an actor is executed 
	 * if this actor is within the current world of the scenario:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>import</b> ...
	 *     <b>import</b> greenfoot.TestRunnerState.WorldCreator;
	 *     
	 *     <i>@RunWith</i>(GreenfootRunner.class)
	 *     <b>public class</b> ExampleRunOnceTest {
	 *         <i>@Test</i>
	 *         <b>public void</b> testRunOnce() <b>throws</b> Exception {
	 *             <b>// GIVEN </b>
	 *             World world1 = WorldCreator.getWorld(15, 15, 60);
	 *             World world2 = WorldCreator.getWorld(15, 15, 60);
	 *             ExampleActorRunOnce actor_1_1 = <b>new</b> ExampleActorRunOnce();
	 *             ExampleActorRunOnce actor_1_2 = <b>new</b> ExampleActorRunOnce();
	 *             world1.addObject(actor_1_1, 3, 5);
	 *             world1.addObject(actor_1_2, 5, 5);
	 *             ExampleActorRunOnce actor_2_1 = <b>new</b> ExampleActorRunOnce();
	 *             ExampleActorRunOnce actor_2_2 = <b>new</b> ExampleActorRunOnce();
	 *             world2.addObject(actor_2_1, 3, 5);
	 *             world2.addObject(actor_2_2, 5, 5);
	 *             
	 *             <b>// WHEN </b>
	 *             WorldCreator.runOnce();
	 *             
	 *             <b>// THEN </b>
	 *             <i>assertEquals</i>( 5, actor_1_1.getX());
	 *             <i>assertEquals</i>( 9, actor_1_2.getX());
	 *             <i>assertEquals</i>( 3, actor_2_1.getX());
	 *             <i>assertEquals</i>( 5, actor_2_2.getX());
	 *         }
	 *     }
	 * </code>
	 * </pre>
     * 
     * @throws GreenfootRunnerExpected is raised when <i>@RunWith</i>(GreenfootRunner.class) 
     *         or <i>@ExtendWith</i>(GreenfootRunner.class) is not found.
     */
    public static void runOnce() throws GreenfootRunnerExpected{
    	TestRunnerState.assertRunningUnitTest();
    	
        beginCycle();
    	World world = WorldHandler.getInstance().getWorld();        	
        if (world != null && !isStop()) {
        	TestRunnerState.setStarted(world);
			world.act();
	        runActorActs(world);
        }
		endCycle();
    }

	private static void runActorActs(World world) throws GreenfootRunnerExpected {
		for (Actor actor: world.getActOrderActors()){
			World currentWorld = WorldHandler.getInstance().getWorld();
		    if (currentWorld == null || currentWorld != world || isStop()) {
		        break;
		        
		    } else if (world.getObjects(Actor.class).contains(actor)) {
				actor.act();
			}
		}
	}
        
    /**
     * It executes one acts cycle (the act of world and the acts 
     * of actors added to world). If this world is not the current 
     * world of the scenario, it is executed Greenfoot.setWorld(world)
     * before acts cycle. Delay, Speed and SoundPlayed events are 
     * deleted before acts are executed and Mic Level, Random and 
     * Ask events are deleted after acts are executed. Also, the 
     * keyboard and mouse buffers are reset after this operation.
     * 
	 * For the next actor behaviour:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>public class</b> ExampleActorRunOnce <b>extends</b> Actor {
	 *         <i>@Override</i>
	 *         <b>public void</b> act(){
	 *             <b>if</b> (isTouching(ExampleActorRunOnce.<b>class</b>)){
	 *                 move(4);
	 *             } <b>else</b> {
	 *                 move(2);
	 *             }
	 *         }
	 *     } 
	 * </code>
	 * </pre>
	 * 
	 * The next test verifies that only the act method of an actor is executed 
	 * if this actor is within the current world of the scenario:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>import</b> ...
	 *     <b>import</b> greenfoot.TestRunnerState.WorldCreator;
	 *     
	 *     <i>@RunWith</i>(GreenfootRunner.class)
	 *     <b>public class</b> ExampleRunOnceTest {
	 *         <i>@Test</i>
	 *         <b>public void</b> testRunOnce() <b>throws</b> Exception {
	 *             <b>// GIVEN </b>
	 *             World world1 = WorldCreator.getWorld(15, 15, 60);
	 *             World world2 = WorldCreator.getWorld(15, 15, 60);
	 *             ExampleActorRunOnce actor_1_1 = <b>new</b> ExampleActorRunOnce();
	 *             ExampleActorRunOnce actor_1_2 = <b>new</b> ExampleActorRunOnce();
	 *             world1.addObject(actor_1_1, 3, 5);
	 *             world1.addObject(actor_1_2, 5, 5);
	 *             ExampleActorRunOnce actor_2_1 = <b>new</b> ExampleActorRunOnce();
	 *             ExampleActorRunOnce actor_2_2 = <b>new</b> ExampleActorRunOnce();
	 *             world2.addObject(actor_2_1, 3, 5);
	 *             world2.addObject(actor_2_2, 5, 5);
	 *             
	 *             <b>// WHEN </b>
	 *             WorldCreator.runOnce(world2);
	 *             
	 *             <b>// THEN </b>
	 *             <i>assertEquals</i>(3, actor_1_1.getX());
	 *             <i>assertEquals</i>(5, actor_1_2.getX());
	 *             <i>assertEquals</i>(5, actor_2_1.getX());
	 *             <i>assertEquals</i>(9, actor_2_2.getX());
	 *         }
	 *     }
	 * </code>
	 * </pre>
     * 
     * @param world where the actors were adding
     * 
     * @throws GreenfootRunnerExpected is raised when <i>@RunWith</i>(GreenfootRunner.class) 
     *         or <i>@ExtendWith</i>(GreenfootRunner.class) is not found.
     */
    public static void runOnce(World world) throws GreenfootRunnerExpected{
    	TestRunnerState.assertRunningUnitTest();
    	
        beginCycle();
    	World currentWorld = WorldHandler.getInstance().getWorld();
    	if (world != null && currentWorld != world) {
    		WorldHandler.getInstance().setWorld(world, true);
    	}
        if (world != null && !isStop()) {
        	TestRunnerState.setStarted(world);
			world.act();
	        runActorActs(world);
        }
		endCycle();
    }
        
    /**
     * It executes one act of the referred actors, that is, of the 
     * actors of the list that is passed as a parameter only when 
     * theirs world is the current world of the scenario. Delay, Speed 
     * and SoundPlayed events are deleted before acts are executed 
     * and Mic Level, Random and Ask events are deleted after acts 
     * are executed. Also, the keyboard and mouse buffers are reset 
     * after this operation.
     * 
	 * <pre>
	 *  <code>
	 *     <b>public class</b> ExampleActorRunOnce <b>extends</b> Actor {
	 *         <i>@Override</i>
	 *         <b>public void</b> act(){
	 *             <b>if</b> (isTouching(ExampleActorRunOnce.<b>class</b>)){
	 *                 move(4);
	 *             } <b>else</b> {
	 *                 move(2);
	 *             }
	 *         }
	 *     } 
	 * </code>
	 * </pre>
	 * 
	 * The next tests verify that only the act method of an actor is executed 
	 * if this actor is within the current world of the scenario:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>import</b> ...
	 *     <b>import</b> greenfoot.TestRunnerState.WorldCreator;
	 *     
	 *     <i>@RunWith</i>(GreenfootRunner.class)
	 *     <b>public class</b> ExampleRunOnceTest {
	 *         World world1;
	 *         World world2;
	 *         ExampleActorRunOnce actor_1_1;
	 *         ExampleActorRunOnce actor_1_2;
	 *         ExampleActorRunOnce actor_1_3;
	 *         ExampleActorRunOnce actor_2_1;
	 *         ExampleActorRunOnce actor_2_2;
	 *         ExampleActorRunOnce actor_2_3;
	 *         
	 *         <i>@Before</i>
	 *         <b>public void</b> setUp() <b>throws</b> Exception {
	 *             <b>// GIVEN </b>
	 *             world1 = WorldCreator.getWorld(15, 15, 60);
	 *             world2 = WorldCreator.getWorld(15, 15, 60);
	 *             actor_1_1 = <b>new</b> ExampleActorRunOnce();
	 *             actor_1_2 = <b>new</b> ExampleActorRunOnce();
	 *             actor_1_3 = <b>new</b> ExampleActorRunOnce();
	 *             world1.addObject(actor_1_1, 3, 5);
	 *             world1.addObject(actor_1_2, 5, 5);
	 *             world1.addObject(actor_1_3, 9, 5);
	 *             actor_2_1 = <b>new</b> ExampleActorRunOnce();
	 *             actor_2_2 = <b>new</b> ExampleActorRunOnce();
	 *             actor_2_3 = <b>new</b> ExampleActorRunOnce();
	 *             world2.addObject(actor_2_1, 3, 5);
	 *             world2.addObject(actor_2_2, 5, 5);
	 *             world1.addObject(actor_2_3, 9, 5);
	 *         }
	 *         
	 *         <i>@Test</i>
	 *         <b>public void</b> testRunOnceOne() <b>throws</b> Exception {
	 *             <b>// WHEN </b>
	 *             WorldCreator.runOnce(actor_1_1);
	 *             
	 *             <b>// THEN </b>
	 *             <i>assertEquals</i>(5, actor_1_1.getX());
	 *             <i>assertEquals</i>(5, actor_1_2.getX());
	 *             <i>assertEquals</i>(9, actor_1_3.getX());
	 *             <i>assertEquals</i>(3, actor_2_1.getX());
	 *             <i>assertEquals</i>(5, actor_2_2.getX());
	 *             <i>assertEquals</i>(9, actor_2_2.getX());
	 *         }
	 *         
	 *         <i>@Test</i>
	 *         <b>public void</b> testRunOnceNone() <b>throws</b> Exception {
	 *             <b>// WHEN </b>
	 *             WorldCreator.runOnce(actor_2_1);
	 *             
	 *             <b>// THEN </b>
	 *             <i>assertEquals</i>(3, actor_1_1.getX());
	 *             <i>assertEquals</i>(5, actor_1_2.getX());
	 *             <i>assertEquals</i>(9, actor_1_3.getX());
	 *             <i>assertEquals</i>(3, actor_2_1.getX());
	 *             <i>assertEquals</i>(5, actor_2_2.getX());
	 *             <i>assertEquals</i>(9, actor_2_2.getX());
	 *         }
	 *         
	 *         <i>@Test</i>
	 *         <b>public void</b> testRunOnceMany() <b>throws</b> Exception {
	 *             <b>// WHEN </b>
	 *             WorldCreator.runOnce(actor_2_1, actor_1_2, actor_2_2, actor_1_1);
	 *             
	 *             <b>// THEN </b>
	 *             <i>assertEquals</i>(5, actor_1_1.getX());
	 *             <i>assertEquals</i>(9, actor_1_2.getX());
	 *             <i>assertEquals</i>(9, actor_1_3.getX());
	 *             <i>assertEquals</i>(3, actor_2_1.getX());
	 *             <i>assertEquals</i>(5, actor_2_2.getX());
	 *             <i>assertEquals</i>(9, actor_2_2.getX());
	 *         }
	 *     }
	 * </code>
	 * </pre>
	 * 
     * @param actors list that may not have been 
     *               added to any world
     *               
     * @throws GreenfootRunnerExpected is raised when <i>@RunWith</i>(GreenfootRunner.class) 
     *         or <i>@ExtendWith</i>(GreenfootRunner.class) is not found.
     */
    public static void runOnce(Actor ... actors) throws GreenfootRunnerExpected {
    	TestRunnerState.assertRunningUnitTest();
        	
    	beginCycle();    	
    	World world = WorldHandler.getInstance().getWorld();
        if (world != null && !isStop()) {
        	TestRunnerState.setStarted(world);
	        runActsList(world, actors);
        }
		endCycle();
    }

	private static void runActsList(World world, Actor... actors) throws GreenfootRunnerExpected {
		for (Actor actor: world.getActOrderActors()){
			World currentWorld = WorldHandler.getInstance().getWorld();
		    if (currentWorld == null || currentWorld != world || isStop()) {
		        break;
		        
		    } else if (world.getObjects(Actor.class).contains(actor)) {
		    	for (Actor actorElement: actors) {
		    		if (actor == actorElement) {
						actor.act();
						break;
		    		}
		    	}
			}
		}
	}
    
    /**
     * It executes an act cycle of the world and referenced actors that 
     * have been added to this world. If this world is not the current 
     * world of the scenario, it is executed Greenfoot.setWorld(world)
     * before acts cycle. Delay, Speed and SoundPlayed events are 
     * deleted before acts are executed and Mic Level, Random and Ask 
     * events are deleted after acts are executed. Also, the keyboard 
     * and mouse buffers are reset after this operation.
     * 
	 * <pre>
	 *  <code>
	 *     <b>public class</b> ExampleActorRunOnce <b>extends</b> Actor {
	 *         <i>@Override</i>
	 *         <b>public void</b> act(){
	 *             <b>if</b> (isTouching(ExampleActorRunOnce.<b>class</b>)){
	 *                 move(4);
	 *             } <b>else</b> {
	 *                 move(2);
	 *             }
	 *         }
	 *     } 
	 * </code>
	 * </pre>
	 * 
	 * The next tests verify that only the act method of an actor is executed 
	 * if this actor is within the current world of the scenario:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>import</b> ...
	 *     <b>import</b> greenfoot.TestRunnerState.WorldCreator;
	 *     
	 *     <i>@RunWith</i>(GreenfootRunner.class)
	 *     <b>public class</b> ExampleRunOnceTest {
	 *         World world1;
	 *         World world2;
	 *         ExampleActorRunOnce actor_1_1;
	 *         ExampleActorRunOnce actor_1_2;
	 *         ExampleActorRunOnce actor_1_3;
	 *         ExampleActorRunOnce actor_2_1;
	 *         ExampleActorRunOnce actor_2_2;
	 *         ExampleActorRunOnce actor_2_3;
	 *         
	 *         <i>@Before</i>
	 *         <b>public void</b> setUp() <b>throws</b> Exception {
	 *             <b>// GIVEN </b>
	 *             world1 = WorldCreator.getWorld(15, 15, 60);
	 *             world2 = WorldCreator.getWorld(15, 15, 60);
	 *             actor_1_1 = <b>new</b> ExampleActorRunOnce();
	 *             actor_1_2 = <b>new</b> ExampleActorRunOnce();
	 *             actor_1_3 = <b>new</b> ExampleActorRunOnce();
	 *             world1.addObject(actor_1_1, 3, 5);
	 *             world1.addObject(actor_1_2, 5, 5);
	 *             world1.addObject(actor_1_3, 9, 5);
	 *             actor_2_1 = <b>new</b> ExampleActorRunOnce();
	 *             actor_2_2 = <b>new</b> ExampleActorRunOnce();
	 *             actor_2_3 = <b>new</b> ExampleActorRunOnce();
	 *             world2.addObject(actor_2_1, 3, 5);
	 *             world2.addObject(actor_2_2, 5, 5);
	 *             world1.addObject(actor_2_3, 9, 5);
	 *         }
	 *         
	 *         <i>@Test</i>
	 *         <b>public void</b> testRunOnceNone() <b>throws</b> Exception {
	 *             <b>// WHEN </b>
	 *             WorldCreator.runOnce(world2, actor_1_1);
	 *             
	 *             <b>// THEN </b>
	 *             <i>assertEquals</i>(3, actor_1_1.getX());
	 *             <i>assertEquals</i>(5, actor_1_2.getX());
	 *             <i>assertEquals</i>(9, actor_1_3.getX());
	 *             <i>assertEquals</i>(3, actor_2_1.getX());
	 *             <i>assertEquals</i>(5, actor_2_2.getX());
	 *             <i>assertEquals</i>(9, actor_2_2.getX());
	 *         }
	 *         
	 *         <i>@Test</i>
	 *         <b>public void</b> testRunOnceOne() <b>throws</b> Exception {
	 *             <b>// WHEN </b>
	 *             WorldCreator.runOnceOne(world2, actor_2_1);
	 *             
	 *             <b>// THEN </b>
	 *             <i>assertEquals</i>(3, actor_1_1.getX());
	 *             <i>assertEquals</i>(5, actor_1_2.getX());
	 *             <i>assertEquals</i>(9, actor_1_3.getX());
	 *             <i>assertEquals</i>(5, actor_2_1.getX());
	 *             <i>assertEquals</i>(5, actor_2_2.getX());
	 *             <i>assertEquals</i>(9, actor_2_2.getX());
	 *         }
	 *         
	 *         <i>@Test</i>
	 *         <b>public void</b> testRunOnceMany() <b>throws</b> Exception {
	 *             <b>// WHEN </b>
	 *             WorldCreator.runOnce(world2, actor_2_1, actor_1_2, actor_2_2, actor_1_1);
	 *             
	 *             <b>// THEN </b>
	 *             <i>assertEquals</i>(3, actor_1_1.getX());
	 *             <i>assertEquals</i>(5, actor_1_2.getX());
	 *             <i>assertEquals</i>(9, actor_1_3.getX());
	 *             <i>assertEquals</i>(5, actor_2_1.getX());
	 *             <i>assertEquals</i>(9, actor_2_2.getX());
	 *             <i>assertEquals</i>(9, actor_2_2.getX());
	 *         }
	 *     }
	 * </code>
	 * </pre>
	 * 
     * @param world where the actors were adding
     * 
     * @param actors list that must be had added to world
     * 
     * @throws GreenfootRunnerExpected is raised when <i>@RunWith</i>(GreenfootRunner.class) 
     *         or <i>@ExtendWith</i>(GreenfootRunner.class) is not found.
     */
    public static void runOnce(World world, Actor ... actors) throws GreenfootRunnerExpected {
    	if (world != null) {
        	TestRunnerState.assertRunningUnitTest();
            	
	    	beginCycle();
        	World currentWorld = WorldHandler.getInstance().getWorld();
        	if (world != null && currentWorld != world) {
        		WorldHandler.getInstance().setWorld(world, true);
        	}
            if (world != null && !isStop()) {
            	TestRunnerState.setStarted(world);
				world.act();
		        runActsList(world, actors);
            }
			endCycle();  
    	}
    }
    
    /**
     * Initialize acts cycle
     */
    private static void beginCycle(){
        TestRunnerState.resetOutputs();  
    }
    
    /**
     * Finalize acts cycle
     */
    private static void endCycle(){
    	try {
			EventDispatch.keyboardExited();
			EventDispatch.mouseExited();
	        TestRunnerState.resetInputs();  
	        
		} catch (GreenfootRunnerExpected e) {
			e.printStackTrace();
		}
    }
    
    /**
     * It verifies if object has paused the animation calling to Greenfoot.Stop(). <br>
	 * <br>
	 * For the next actor behaviour:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>public class</b> ExampleActorStop <b>extends</b> Actor {
	 *     
	 *         <i>@Override</i>
	 *         <b>public void</b> act(){
	 *             Greenfoot.stop();
	 *         }
	 *     } 
	 * </code>
	 * </pre>
	 * 
	 * The next test verifies that the animation was stop:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>import</b> ...
	 *     <b>import</b> greenfoot.TestRunnerState.WorldCreator;
	 *     
	 *     <i>@RunWith</i>(GreenfootRunner.class)
	 *     <b>public class</b> ExampleActorStopTest {
	 *         <i>@Test</i>
	 *         <b>public void</b> testStop() <b>throws</b> Exception {
	 *             <b>// GIVEN </b>
	 *             World world = WorldCreator.getWorld(400, 300, 1);
	 *             ExampleActorStop actor = <b>new</b> ExampleActorStop();
	 *             world.addObject(actor, 1, 1);
	 *             
	 *             <b>// WHEN </b>
	 *             WorldCreator.runOnce(world);
	 *             
	 *             <b>// THEN </b>
	 *             <i>assertTrue</i>(WorldCreator.isStop());
	 *         }
	 *     }
	 * </code>
	 * </pre>
     * 
     * @return true when it is paused
     * 
     * @throws GreenfootRunnerExpected is raised when <i>@RunWith</i>(GreenfootRunner.class) 
     *         or <i>@ExtendWith</i>(GreenfootRunner.class) is not found.
     */
    public static boolean isStop() throws GreenfootRunnerExpected {
    	TestRunnerState.assertRunningUnitTest();
        return TestRunnerState.isStop();
    }

    /**
     * It verifies if any object has start the animation calling to Greenfoot.Start(). <br>
	 * <br>
	 * For the next actor behaviour:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>public class</b> ExampleActorStart <b>extends</b> Actor {
	 *     
	 *         <i>@Override</i>
	 *         <b>public void</b> act(){
	 *             Greenfoot.start();
	 *         }
	 *     } 
	 * </code>
	 * </pre>
	 * 
	 * The next test verifies that the animation was started:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>import</b> ...
	 *     <b>import</b> greenfoot.TestRunnerState.WorldCreator;
	 *     
	 *     <i>@RunWith</i>(GreenfootRunner.class)
	 *     <b>public class</b> ExampleActorStartTest {
	 *         <i>@Test</i>
	 *         <b>public void</b> testStart() <b>throws</b> Exception {
	 *             <b>// GIVEN </b>
	 *             World world = WorldCreator.getWorld(400, 300, 1);
	 *             ExampleActorStart actor = <b>new</b> ExampleActorStart();
	 *             world.addObject(actor, 1, 1);
	 *             
	 *             <b>// WHEN </b>
	 *             WorldCreator.runOnce(world);
	 *             
	 *             <b>// THEN </b>
	 *             <i>assertTrue</i>(WorldCreator.isStart());
	 *         }
	 *     }
	 * </code>
	 * </pre>
     * 
     * @return true when it is paused
     * 
     * @throws GreenfootRunnerExpected is raised when <i>@RunWith</i>(GreenfootRunner.class) 
     *         or <i>@ExtendWith</i>(GreenfootRunner.class) is not found.
     */
    public static boolean isStart() throws GreenfootRunnerExpected {
    	TestRunnerState.assertRunningUnitTest();
        return TestRunnerState.isStart();
    }

    /**
     * It queues a microphone level in FIFO order for any next Greenfoot.getMicLevel() operation.<br> 
     * <br>
	 * For the next actor behaviour:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>public class</b> ExampleActorMicLevel <b>extends</b> Actor {
	 *     
	 *         <i>@Override</i>
	 *         <b>public void</b> act(){
	 *             <b>if</b> (Greenfoot.getMiLevel() == 90) {
	 *                 move(10);
	 *             }
	 *         }
	 *     } 
	 * </code>
	 * </pre>
	 * 
	 * The next test verifies that a microphone level produces the expected behavior:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>import</b> ...
	 *     <b>import</b> greenfoot.TestRunnerState.WorldCreator;
	 *     
	 *     <i>@RunWith</i>(GreenfootRunner.class)
	 *     <b>public class</b> ExampleActorMicLevelTest {
	 *         <i>@Test</i>
	 *         <b>public void</b> testMicLevel() <b>throws</b> Exception {
	 *             <b>// GIVEN </b>
	 *             World world = WorldCreator.getWorld(400, 300, 1);
	 *             ExampleActorMicLevel actor = <b>new</b> ExampleActorMicLevel();
	 *             world.addObject(actor, 1, 1);
	 *             
	 *             <b>// WHEN </b>
	 *             WorldCreator.setMicLevel(90);
	 *             WorldCreator.runOnce(world);
	 *             
	 *             <b>// THEN </b>
	 *             <i>assertEquals</i>(11, WorldCreator.getX());
	 *         }
	 *     }
	 * </code>
	 * </pre>
     * 
     * @param level is the value tested
     * 
     * @throws GreenfootRunnerExpected is raised when <i>@RunWith</i>(GreenfootRunner.class) 
     *         or <i>@ExtendWith</i>(GreenfootRunner.class) is not found.
     * 
     */
    public static void setMicLevel(int level) throws GreenfootRunnerExpected {
    	TestRunnerState.assertRunningUnitTest();
        TestRunnerState.setMicLevel(level);
    }

    /**
     * It return the speed of the animation that was set by any object. <br>
	 * <br>
	 * For the next actor behaviour:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>public class</b> ExampleActorSpeed <b>extends</b> Actor {
	 *     
	 *         <i>@Override</i>
	 *         <b>public void</b> act(){
	 *             Greenfoot.setSpeed(100);
	 *         }
	 *     } 
	 * </code>
	 * </pre>
	 * 
	 * The next test verifies that the speed that has been set is the expected:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>import</b> ...
	 *     <b>import</b> greenfoot.TestRunnerState.WorldCreator;
	 *     
	 *     <i>@RunWith</i>(GreenfootRunner.class)
	 *     <b>public class</b> ExampleActorSpeedTest {
	 *         <i>@Test</i>
	 *         <b>public void</b> testSpeed() <b>throws</b> Exception {
	 *             <b>// GIVEN </b>
	 *             World world = WorldCreator.getWorld(400, 300, 1);
	 *             ExampleActorSpeed actor = <b>new</b> ExampleActorSpeed();
	 *             world.addObject(actor, 1, 1);
	 *             
	 *             <b>// WHEN </b>
	 *             WorldCreator.runOnce(world);
	 *             
	 *             <b>// THEN </b>
	 *             <i>assertEquals</i>(100, WorldCreator.getSpeed());
	 *         }
	 *     }
	 * </code>
	 * </pre>
     * 
     * @return next speed was done
     * 
     * @throws GreenfootRunnerExpected is raised when <i>@RunWith</i>(GreenfootRunner.class) 
     *         or <i>@ExtendWith</i>(GreenfootRunner.class) is not found.
     * 
     */
    public static int getSpeed() throws GreenfootRunnerExpected {
    	TestRunnerState.assertRunningUnitTest();
        return TestRunnerState.getSpeed();
    }

    /**
     * It verifies that any object has set this speed of the animation. <br>
	 * <br>
	 * For the next actor behaviour:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>public class</b> ExampleActorSpeed <b>extends</b> Actor {
	 *     
	 *         <i>@Override</i>
	 *         <b>public void</b> act(){
	 *             Greenfoot.setSpeed(100);
	 *         }
	 *     } 
	 * </code>
	 * </pre>
	 * 
	 * The next test verifies that the speed that has been set is the expected:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>import</b> ...
	 *     <b>import</b> greenfoot.TestRunnerState.WorldCreator;
	 *     
	 *     <i>@RunWith</i>(GreenfootRunner.class)
	 *     <b>public class</b> ExampleActorSpeedTest {
	 *         <i>@Test</i>
	 *         <b>public void</b> testSpeed() <b>throws</b> Exception {
	 *             <b>// GIVEN </b>
	 *             World world = WorldCreator.getWorld(400, 300, 1);
	 *             ExampleActorSpeed actor = <b>new</b> ExampleActorSpeed();
	 *             world.addObject(actor, 1, 1);
	 *             
	 *             <b>// WHEN </b>
	 *             WorldCreator.runOnce(world);
	 *             
	 *             <b>// THEN </b>
	 *             <i>assertTrue</i>(WorldCreator.getSpeed(100));
	 *         }
	 *     }
	 * </code>
	 * </pre>
     * 
     * @param speed is the value tested
     * 
     * @return true when any world or actor has set this speed
     * 
     * @throws GreenfootRunnerExpected is raised when <i>@RunWith</i>(GreenfootRunner.class) 
     *         or <i>@ExtendWith</i>(GreenfootRunner.class) is not found.
     * 
     */
    public static boolean getSpeed(int speed) throws GreenfootRunnerExpected {
    	TestRunnerState.assertRunningUnitTest();
        return TestRunnerState.getSpeed(speed);
    }

    /**
     * It returns the delay was set by any object. <br>
	 * <br>
	 * For the next actor behaviour:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>public class</b> ExampleActorDelay <b>extends</b> Actor {
	 *     
	 *         <i>@Override</i>
	 *         <b>public void</b> act(){
	 *             Greenfoot.setDelay(12);
	 *         }
	 *     } 
	 * </code>
	 * </pre>
	 * 
	 * The next test verifies that the delay that has been set is the expected:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>import</b> ...
	 *     <b>import</b> greenfoot.TestRunnerState.WorldCreator;
	 *     
	 *     <i>@RunWith</i>(GreenfootRunner.class)
	 *     <b>public class</b> ExampleActorDelayTest {
	 *         <i>@Test</i>
	 *         <b>public void</b> testDelay() <b>throws</b> Exception {
	 *             <b>// GIVEN </b>
	 *             World world = WorldCreator.getWorld(400, 300, 1);
	 *             ExampleActorDelay actor = <b>new</b> ExampleActorDelay();
	 *             world.addObject(actor, 1, 1);
	 *             
	 *             <b>// WHEN </b>
	 *             WorldCreator.runOnce(world);
	 *             
	 *             <b>// THEN </b>
	 *             <i>assertEquals</i>(12, WorldCreator.getDelay());
	 *         }
	 *     }
	 * </code>
	 * </pre>
     * 
     * @return next delay that was done
     * 
     * @throws GreenfootRunnerExpected is raised when <i>@RunWith</i>(GreenfootRunner.class) 
     *         or <i>@ExtendWith</i>(GreenfootRunner.class) is not found.
     */
    public static int getDelay() throws GreenfootRunnerExpected {
    	TestRunnerState.assertRunningUnitTest();
        return TestRunnerState.getDelay();
    }

    /**
     * It verifies any object has set a delay. <br>
	 * <br>
	 * For the next actor behaviour:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>public class</b> ExampleActorDelay <b>extends</b> Actor {
	 *     
	 *         <i>@Override</i>
	 *         <b>public void</b> act(){
	 *             Greenfoot.setDelay(12);
	 *         }
	 *     } 
	 * </code>
	 * </pre>
	 * 
	 * The next test verifies that the delay that has been set is the expected:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>import</b> ...
	 *     <b>import</b> greenfoot.TestRunnerState.WorldCreator;
	 *     
	 *     <i>@RunWith</i>(GreenfootRunner.class)
	 *     <b>public class</b> ExampleActorDelayTest {
	 *         <i>@Test</i>
	 *         <b>public void</b> testDelay() <b>throws</b> Exception {
	 *             <b>// GIVEN </b>
	 *             World world = WorldCreator.getWorld(400, 300, 1);
	 *             ExampleActorDelay actor = <b>new</b> ExampleActorDelay();
	 *             world.addObject(actor, 1, 1);
	 *             
	 *             <b>// WHEN </b>
	 *             WorldCreator.runOnce(world);
	 *             
	 *             <b>// THEN </b>
	 *             <i>assertTrue</i>(WorldCreator.getDelay());
	 *         }
	 *     }
	 * </code>
	 * </pre>
     * 
     * @param delay is the value tested
     * 
     * @return true when any object set this delay
     * 
     * @throws GreenfootRunnerExpected is raised when <i>@RunWith</i>(GreenfootRunner.class) 
     *         or <i>@ExtendWith</i>(GreenfootRunner.class) is not found.
     */
    public static boolean getDelay(int delay) throws GreenfootRunnerExpected {
    	TestRunnerState.assertRunningUnitTest();
        return TestRunnerState.getDelay(delay);
    }
}
